import cv2


from skellytracker.trackers.bright_point_tracker.brightest_point_tracker import (
    BrightestPointTracker,
)
from skellytracker.trackers.charuco_tracker.charuco_tracker import CharucoTracker
try:
    from skellytracker.trackers.mediapipe_tracker.mediapipe_holistic_tracker import (
        MediapipeHolisticTracker,
    )
except:
    print("To use mediapipe_holistic_tracker, install skellytracker[mediapipe]")
try:
    from skellytracker.trackers.yolo_tracker.yolo_tracker import YOLOPoseTracker
    from skellytracker.trackers.yolo_object_tracker.yolo_object_tracker import (
        YOLOObjectTracker,
    )
    from skellytracker.trackers.segment_anything_tracker.segment_anything_tracker import (
        SAMTracker,
    )
except:
    print("To use yolo_tracker, install skellytracker[yolo]")


def main(demo_tracker: str = "mediapipe_holistic_tracker"):

    if demo_tracker == "brightest_point_tracker":
        BrightestPointTracker().demo()

    elif demo_tracker == "charuco_tracker":
        CharucoTracker(
            squaresX=7,
            squaresY=5,
            dictionary=cv2.aruco.getPredefinedDictionary(cv2.aruco.DICT_4X4_250),
        ).demo()

    elif demo_tracker == "mediapipe_holistic_tracker":
        MediapipeHolisticTracker(
            model_complexity=2,
            min_detection_confidence=0.5,
            min_tracking_confidence=0.5,
            static_image_mode=False,
            smooth_landmarks=True,
        ).demo()

    elif demo_tracker == "yolo_tracker":
        YOLOPoseTracker(model_size="nano").demo()
    elif demo_tracker == "SAM_tracker":
        SAMTracker().demo()
    elif demo_tracker == "yolo_object_tracker":
        YOLOObjectTracker(model_size="medium").demo()
        
if __name__ == "__main__":
    main(demo_tracker="mediapipe_holistic_tracker")


import cv2
import mediapipe as mp
import numpy as np
from typing import Dict

from skellytracker.trackers.base_tracker.base_tracker import BaseTracker
from skellytracker.trackers.base_tracker.tracked_object import TrackedObject
from skellytracker.trackers.mediapipe_tracker.mediapipe_holistic_recorder import (
    MediapipeHolisticRecorder,
)
from skellytracker.trackers.mediapipe_tracker.mediapipe_model_info import (
    MediapipeModelInfo,
)


class MediapipeHolisticTracker(BaseTracker):
    def __init__(
        self,
        model_complexity=2,
        min_detection_confidence=0.5,
        min_tracking_confidence=0.5,
        static_image_mode=False,
        smooth_landmarks=True,
    ):
        super().__init__(
            tracked_object_names=MediapipeModelInfo.mediapipe_tracked_object_names,
            recorder=MediapipeHolisticRecorder(),
        )
        self.mp_drawing = mp.solutions.drawing_utils
        self.mp_holistic = mp.solutions.holistic
        self.holistic = self.mp_holistic.Holistic(
            model_complexity=model_complexity,
            min_detection_confidence=min_detection_confidence,
            min_tracking_confidence=min_tracking_confidence,
            static_image_mode=static_image_mode,
            smooth_landmarks=smooth_landmarks,
        )

    def process_image(self, image: np.ndarray, **kwargs) -> Dict[str, TrackedObject]:
        # Convert the image to RGB
        rgb_image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)

        # Process the image
        results = self.holistic.process(rgb_image)

        # Update the tracking data
        self.tracked_objects["pose_landmarks"].extra[
            "landmarks"
        ] = results.pose_landmarks
        self.tracked_objects["face_landmarks"].extra[
            "landmarks"
        ] = results.face_landmarks
        self.tracked_objects["left_hand_landmarks"].extra[
            "landmarks"
        ] = results.left_hand_landmarks
        self.tracked_objects["right_hand_landmarks"].extra[
            "landmarks"
        ] = results.right_hand_landmarks

        self.raw_image = image.copy()

        self.annotated_image = self.annotate_image(
            image=image, tracked_objects=self.tracked_objects
        )

        return self.tracked_objects

    def annotate_image(
        self, image: np.ndarray, tracked_objects: Dict[str, TrackedObject], **kwargs
    ) -> np.ndarray:
        # Draw the pose, face, and hand landmarks on the image
        self.mp_drawing.draw_landmarks(
            image,
            tracked_objects["pose_landmarks"].extra["landmarks"],
            self.mp_holistic.POSE_CONNECTIONS,
        )
        self.mp_drawing.draw_landmarks(
            image,
            tracked_objects["face_landmarks"].extra["landmarks"],
            self.mp_holistic.FACEMESH_TESSELATION,
        )
        self.mp_drawing.draw_landmarks(
            image,
            tracked_objects["left_hand_landmarks"].extra["landmarks"],
            self.mp_holistic.HAND_CONNECTIONS,
        )
        self.mp_drawing.draw_landmarks(
            image,
            tracked_objects["right_hand_landmarks"].extra["landmarks"],
            self.mp_holistic.HAND_CONNECTIONS,
        )

        return image


if __name__ == "__main__":
    MediapipeHolisticTracker(
        model_complexity=2,
        min_detection_confidence=0.5,
        min_tracking_confidence=0.5,
        static_image_mode=False,
        smooth_landmarks=True,
    ).demo()



"""Top-level package for skellytracker"""

__package_name__ = "skellytracker"
__version__ = "v2024.04.1014"

__author__ = """Skelly FreeMoCap"""
__email__ = "info@freemocap.org"
__repo_owner_github_user_name__ = "freemocap"
__repo_url__ = f"https://github.com/{__repo_owner_github_user_name__}/{__package_name__}"
__repo_issues_url__ = f"{__repo_url__}/issues"

import sys
from pathlib import Path

print(f"Thank you for using {__package_name__}!")
print(f"This is printing from: {__file__}")
print(f"Source code for this package is available at: {__repo_url__}")

base_package_path = Path(__file__).parent
print(f"adding base_package_path: {base_package_path} : to sys.path")
sys.path.insert(0, str(base_package_path))  # add parent directory to sys.path

from skellytracker.system.default_paths import get_log_file_path
from skellytracker.system.logging_configuration import configure_logging

try:
    from skellytracker.trackers.mediapipe_tracker.mediapipe_holistic_tracker import MediapipeHolisticTracker
except:
    print("To use mediapipe_holistic_tracker, install skellytracker[mediapipe]")
try:
    from skellytracker.trackers.yolo_tracker.yolo_tracker import YOLOPoseTracker
except:
    print("To use yolo_tracker, install skellytracker[yolo]")
try:
    from skellytracker.trackers.yolo_mediapipe_combo_tracker.yolo_mediapipe_combo_tracker import YOLOMediapipeComboTracker
except:
    print("To use yolo_mediapipe_combo_tracker, install skellytracker[mediapipe, yolo] or skellytracker[all]")



configure_logging(log_file_path=get_log_file_path())


from skellytracker.trackers.base_tracker.base_tracking_params import BaseTrackingParams


class YOLOModelInfo:
    num_tracked_points = 17
    model_dictionary = {
        "nano": "yolov8n-pose.pt",
        "small": "yolov8s-pose.pt",
        "medium": "yolov8m-pose.pt",
        "large": "yolov8l-pose.pt",
        "extra_large": "yolov8x-pose.pt",
        "high_res": "yolov8x-pose-p6.pt",
    }
    marker_dict = {
        0: "nose",
        1: "left_eye",
        2: "right_eye",
        3: "left_ear",
        4: "right_ear",
        5: "left_shoulder",
        6: "right_shoulder",
        7: "left_elbow",
        8: "right_elbow",
        9: "left_wrist",
        10: "right_wrist",
        11: "left_hip",
        12: "right_hip",
        13: "left_knee",
        14: "right_knee",
        15: "left_ankle",
        16: "right_ankle",
    }
    body_segment_names = [
        "head",
        "trunk",
        "right_upper_arm",
        "left_upper_arm",
        "right_forearm",
        "left_forearm",
        "right_thigh",
        "left_thigh",
        "right_shin",
        "left_shin",
    ]
    joint_connections = [
        ["left_ear", "right_ear"],
        ["mid_chest_marker", "mid_hip_marker"],
        ["right_shoulder", "right_elbow"],
        ["left_shoulder", "left_elbow"],
        ["right_elbow", "right_wrist"],
        ["left_elbow", "left_wrist"],
        ["right_hip", "right_knee"],
        ["left_hip", "left_knee"],
        ["right_knee", "right_ankle"],
        ["left_knee", "left_ankle"],
    ]
    segment_COM_lengths = [
        0.5,
        0.5,
        0.436,
        0.436,
        0.682,
        0.682,
        0.433,
        0.433,
        0.606,
        0.606,
    ]
    segment_COM_percentages = [
        0.081,
        0.497,
        0.028,
        0.028,
        0.022,
        0.022,
        0.1,
        0.1,
        0.061,
        0.061,
    ]


class YOLOTrackingParams(BaseTrackingParams):
    model_size: str = "medium"



import multiprocessing as mp
import time
from asyncio import sleep
from typing import List

from skellytracker.trackers.base_tracker.base_tracker import BaseTracker
from skellytracker.trackers.bright_point_tracker.brightest_point_tracker import BrightestPointTracker


class TrackerManager:
    def __init__(self, trackers: List[BaseTracker]):
        self.trackers = trackers
        self.parent_connection, self.child_connection = mp.Pipe()
        self.process = mp.Process(target=self._process_images, args=(self.child_connection, self.trackers))
        self.process.start()

    @staticmethod
    def _process_images(conn, trackers):
        while True:
            time.sleep(0.001)
            image = conn.recv()
            if image is None:
                break
            for tracker in trackers:
                tracker.process_image(image)

    def add_image(self, image):
        self.parent_connection.send(image)

    def demo(self):
        self.trackers[0].demo()

    def stop(self):
        self.parent_connection.send(None)
        self.process.join()



if __name__ == "__main__":

    trackers = [BrightestPointTracker()]

    manager = TrackerManager(trackers)
    manager.demo()



import logging
from pathlib import Path
from typing import Union
import cv2
import numpy as np

logger = logging.getLogger(__name__)


class VideoHandler:
    def __init__(
        self,
        output_path: Union[Path, str],
        frame_size: tuple[int, int],
        fps: float = 30.0,
        codec: str = "mp4v",
    ):
        """
        Initialize the VideoHandler.

        :param output_path: The path to the output video file.
        :param frame_size: The size of the frames (width, height).
        :param fps: The frames per second of the output video.
        :param codec: The codec to use for the output video.
        """
        self.output_path = output_path
        fourcc = cv2.VideoWriter_fourcc(*codec)
        self.video_writer = cv2.VideoWriter(
            str(output_path), fourcc, fps, frame_size
        )

    def add_frame(self, frame: np.ndarray) -> None:
        """
        Add a frame to the video.

        :param frame: The frame to add.
        """
        self.video_writer.write(frame)

    def close(self):
        """
        Close the video file.
        """
        self.video_writer.release()
        logger.info(f"video saved to {self.output_path}")



from typing import Dict, List

import cv2
import numpy as np

from skellytracker.trackers.base_tracker.base_tracker import BaseTracker
from skellytracker.trackers.base_tracker.tracked_object import TrackedObject


class CharucoTracker(BaseTracker):
    def __init__(self,
                 tracked_object_names: List[str],
                 squares_x: int,
                 squares_y: int,
                 dictionary: cv2.aruco_Dictionary,
                 squareLength: float = 1,
                 markerLength: float = .8,
                 ):
        super().__init__(tracked_object_names=tracked_object_names)
        self.board = cv2.aruco.CharucoBoard_create(squares_x, squares_y, squareLength, markerLength, dictionary)

    def process_image(self, image: np.ndarray, **kwargs) -> Dict[str, TrackedObject]:
        # Convert the image to grayscale
        gray_image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

        # Detect Aruco markers
        corners, ids, _ = cv2.aruco.detectMarkers(gray_image, self.board.dictionary)

        # If any markers were found
        if len(corners) > 0:
            # Refine the detected markers
            ret, charuco_corners, charuco_ids = cv2.aruco.interpolateCornersCharuco(corners, ids, gray_image,
                                                                                    self.board)

            # If any Charuco corners were found
            if charuco_corners is not None and charuco_ids is not None and len(charuco_corners) > 3:
                # Clear previous tracked objects
                self.tracked_objects.clear()

                # Create a TrackedObject for each corner
                for i, corner in enumerate(charuco_corners):
                    object_id = str(i)
                    self.tracked_objects[object_id] = TrackedObject(object_id=object_id)
                    self.tracked_objects[object_id].pixel_x = corner[0][0]
                    self.tracked_objects[object_id].pixel_y = corner[0][1]

        self.raw_image = image.copy()

        self.annotated_image = self.annotate_image(image=image,
                                                   tracked_objects=self.tracked_objects)

        return self.tracked_objects

    def annotate_image(self, image: np.ndarray, tracked_objects: Dict[str, TrackedObject], **kwargs) -> np.ndarray:
        # Copy the original image for annotation
        annotated_image = image.copy()

        # Draw a marker for each tracked corner
        for tracked_object in tracked_objects.values():
            if tracked_object.pixel_x is not None and tracked_object.pixel_y is not None:
                cv2.drawMarker(annotated_image,
                               (int(tracked_object.pixel_x), int(tracked_object.pixel_y)),
                               (0, 0, 255), markerType=cv2.MARKER_CROSS, markerSize=20, thickness=2)

        return annotated_image


if __name__ == "__main__":
    charuco_squares_x_in = 7
    charuco_squares_y_in = 5
    number_of_charuco_markers = charuco_squares_x_in - 1 * charuco_squares_y_in - 1
    charuco_ids = [str(index) for index in range(number_of_charuco_markers)]

    CharucoTracker(tracked_object_names=charuco_ids,
                   squares_x=charuco_squares_x_in,
                   squares_y=charuco_squares_y_in,
                   dictionary=cv2.aruco.Dictionary_get(cv2.aruco.DICT_4X4_250)
                   ).demo()



import cv2
import numpy as np
import copy
import mediapipe as mp
import torch
from typing import Dict, Literal, Tuple
from ultralytics import YOLO

from skellytracker.trackers.base_tracker.base_tracker import BaseTracker
from skellytracker.trackers.base_tracker.tracked_object import TrackedObject
from skellytracker.trackers.mediapipe_tracker.mediapipe_holistic_recorder import (
    MediapipeHolisticRecorder,
)
from skellytracker.trackers.mediapipe_tracker.mediapipe_model_info import (
    MediapipeModelInfo,
)
from skellytracker.trackers.yolo_object_tracker.yolo_object_model_info import (
    yolo_object_model_dictionary,
)


class YOLOMediapipeComboTracker(BaseTracker):
    def __init__(
        self,
        model_size: str = "nano",
        model_complexity: int = 2,
        min_detection_confidence: float = 0.5,
        min_tracking_confidence: float = 0.5,
        static_image_mode: bool = False,
        smooth_landmarks: bool = True,
        bounding_box_buffer_percentage: float = 10,
        buffer_size_method: Literal[
            "buffer_by_box_size", "buffer_by_image_size"
        ] = "buffer_by_box_size",
    ):
        super().__init__(
            tracked_object_names=MediapipeModelInfo.mediapipe_tracked_object_names,
            recorder=MediapipeHolisticRecorder(),
        )
        self.mp_drawing = mp.solutions.drawing_utils
        self.mp_holistic = mp.solutions.holistic
        self.holistic = self.mp_holistic.Holistic(
            model_complexity=model_complexity,
            min_detection_confidence=min_detection_confidence,
            min_tracking_confidence=min_tracking_confidence,
            static_image_mode=static_image_mode,
            smooth_landmarks=smooth_landmarks,
        )

        pytorch_model = yolo_object_model_dictionary[model_size]
        self.model = YOLO(pytorch_model)
        self.bounding_box_buffer_percentage = bounding_box_buffer_percentage
        self.buffer_size_method = buffer_size_method

    def process_image(self, image: np.ndarray, **kwargs) -> Dict[str, TrackedObject]:

        yolo_results = self.model(image, classes=0, max_det=1, verbose=False)
        box_xyxy = np.asarray(yolo_results[0].boxes.xyxy).flatten()

        if box_xyxy.size > 0:
            box_left, box_top, box_right, box_bottom = box_xyxy

            if self.buffer_size_method == "buffer_by_image_size":
                width_buffer, height_buffer = self._get_buffer_bounding_box_total_image(
                    image, self.bounding_box_buffer_percentage
                )
            elif self.buffer_size_method == "buffer_by_box_size":
                width_buffer, height_buffer = self._get_buffer_bounding_box_box_size(
                    box_xyxy, self.bounding_box_buffer_percentage
                )
            else:
                raise ValueError(
                    f"Unknown buffer_size_method: {self.buffer_size_method}"
                )

            # Apply buffer, but set to original picture dimension if it goes out of bounds
            box_left = max(int(box_left - width_buffer), 0)
            box_top = max(int(box_top - height_buffer), 0)
            box_right = min(int(box_right + width_buffer), image.shape[1])
            box_bottom = min(int(box_bottom + height_buffer), image.shape[0])

            cropped_image = image[
                int(box_top) : int(box_bottom),
                int(box_left) : int(box_right),
            ]

            buffered_yolo_results = copy.deepcopy(yolo_results)
            buffered_yolo_results[0].boxes.xyxy[0] = torch.tensor(
                [box_left, box_top, box_right, box_bottom]
            )

        else:
            # eventually we should not even run mediapipe if no bbox is found
            box_left, box_top, box_right, box_bottom = 0, 0, 0, 0
            cropped_image = image

            buffered_yolo_results = yolo_results

        cropped_rgb_image = cv2.cvtColor(cropped_image, cv2.COLOR_BGR2RGB)

        mediapipe_results = self.holistic.process(cropped_rgb_image)

        # Update the tracking data
        self._rescale_cropped_data(
            image, box_left, box_top, box_right, box_bottom, mediapipe_results
        )
        self.tracked_objects["pose_landmarks"].extra[
            "landmarks"
        ] = mediapipe_results.pose_landmarks
        self.tracked_objects["face_landmarks"].extra[
            "landmarks"
        ] = mediapipe_results.face_landmarks
        self.tracked_objects["left_hand_landmarks"].extra[
            "landmarks"
        ] = mediapipe_results.left_hand_landmarks
        self.tracked_objects["right_hand_landmarks"].extra[
            "landmarks"
        ] = mediapipe_results.right_hand_landmarks

        bbox_image = buffered_yolo_results[0].plot()

        self.annotated_image = self.annotate_image(
            image=bbox_image, tracked_objects=self.tracked_objects
        )

        return self.tracked_objects

    def _get_buffer_bounding_box_total_image(
        self, image: np.ndarray, buffer_percentage: float
    ) -> Tuple[float, float]:
        width_buffer = image.shape[1] * (buffer_percentage / 100.0)
        height_buffer = image.shape[0] * (buffer_percentage / 100.0)

        return width_buffer, height_buffer

    def _get_buffer_bounding_box_box_size(
        self, box_xyxy: np.ndarray, buffer_percentage: float
    ) -> Tuple[float, float]:
        box_left, box_top, box_right, box_bottom = box_xyxy
        width_buffer = (box_right - box_left) * (buffer_percentage / 100.0)
        height_buffer = (box_bottom - box_top) * (buffer_percentage / 100.0)

        return width_buffer, height_buffer

    def _rescale_cropped_data(
        self,
        image: np.ndarray,
        box_left: int,
        box_top: int,
        box_right: int,
        box_bottom: int,
        mediapipe_results,
    ) -> None:
        if mediapipe_results.pose_landmarks is not None:
            for landmark in mediapipe_results.pose_landmarks.landmark:
                landmark.x = (
                    (landmark.x * (box_right - box_left)) + box_left
                ) / image.shape[1]
                landmark.y = (
                    (landmark.y * (box_bottom - box_top)) + box_top
                ) / image.shape[0]
                landmark.z = (landmark.z * (box_right - box_left)) / image.shape[1]
        if mediapipe_results.face_landmarks is not None:
            for landmark in mediapipe_results.face_landmarks.landmark:
                landmark.x = (
                    (landmark.x * (box_right - box_left)) + box_left
                ) / image.shape[1]
                landmark.y = (
                    (landmark.y * (box_bottom - box_top)) + box_top
                ) / image.shape[0]
                landmark.z = (landmark.z * (box_right - box_left)) / image.shape[1]
        if mediapipe_results.left_hand_landmarks is not None:
            for landmark in mediapipe_results.left_hand_landmarks.landmark:
                landmark.x = (
                    (landmark.x * (box_right - box_left)) + box_left
                ) / image.shape[1]
                landmark.y = (
                    (landmark.y * (box_bottom - box_top)) + box_top
                ) / image.shape[0]
                landmark.z = (landmark.z * (box_right - box_left)) / image.shape[1]
        if mediapipe_results.right_hand_landmarks is not None:
            for landmark in mediapipe_results.right_hand_landmarks.landmark:
                landmark.x = (
                    (landmark.x * (box_right - box_left)) + box_left
                ) / image.shape[1]
                landmark.y = (
                    (landmark.y * (box_bottom - box_top)) + box_top
                ) / image.shape[0]
                landmark.z = (landmark.z * (box_right - box_left)) / image.shape[1]

    def annotate_image(
        self, image: np.ndarray, tracked_objects: Dict[str, TrackedObject], **kwargs
    ) -> np.ndarray:
        # Draw the pose, face, and hand landmarks on the image
        self.mp_drawing.draw_landmarks(
            image,
            tracked_objects["pose_landmarks"].extra["landmarks"],
            self.mp_holistic.POSE_CONNECTIONS,
        )
        self.mp_drawing.draw_landmarks(
            image,
            tracked_objects["face_landmarks"].extra["landmarks"],
            self.mp_holistic.FACEMESH_TESSELATION,
        )
        self.mp_drawing.draw_landmarks(
            image,
            tracked_objects["left_hand_landmarks"].extra["landmarks"],
            self.mp_holistic.HAND_CONNECTIONS,
        )
        self.mp_drawing.draw_landmarks(
            image,
            tracked_objects["right_hand_landmarks"].extra["landmarks"],
            self.mp_holistic.HAND_CONNECTIONS,
        )

        return image


if __name__ == "__main__":
    YOLOMediapipeComboTracker().demo()
